(* TYPES POUR L'ARBRE DE SYNTAXE ABSTRAITE *)

type exp_int =
  | Val_r1 (* valeur du registre 1 *)
  | Val_r2 (* valeur du registre 2 *)
  | Val of int (* entier *)
  | Sum_op of exp_int * exp_int (* somme des deux arguments *)
  | Prod_op of exp_int * exp_int (* produit des deux arguments *)

type exp_bool =
  | And_op of exp_bool * exp_bool (* argument 1 ET argument 2 *)
  | Or_op of exp_bool * exp_bool (* argument 1 OU argument 2 *)
  | Equal_op of exp_int * exp_int (* egalite entre les arguments *)
  | Inf_op of exp_int * exp_int (* vrai si argument 1 < argument 2 *)
  | Val_true (* vrai *)
  | Val_false (* faux *)

type ast = 
  | While_loop of exp_bool * ast (* boucle "while", argument 1 : condition, argument 2 : corps *)
  | If_cond of exp_bool * ast * ast (* conditionnelle, argument 1 : condition, argument 2 : alors, argument 3 : sinon *)
  | Affect_r1 of exp_int (* affectation dans le registre 1 *)
  | Affect_r2 of exp_int (* affectation dans le registre 2 *)
  | Seq of ast * ast (* sequence d'instructions *)

val eval_ast : ast -> int -> int -> int * int (* interprete un ast, argument 1 : ast, argument 2 : valeur initiale de r1, argument 3 : valeur initiale de r2, resultat : valeurs finales de r1 et r2 *)
